"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_protocol_1 = require("vscode-languageserver-protocol");
const vscode_languageserver_1 = require("vscode-languageserver");
const stream_1 = require("stream");
const vls_1 = require("./services/vls");
const initParams_1 = require("./initParams");
const fs = require("fs");
const vscode_uri_1 = require("vscode-uri");
const glob = require("glob");
const path = require("path");
class NullLogger {
    error(_message) { }
    warn(_message) { }
    info(_message) { }
    log(_message) { }
}
class TestStream extends stream_1.Duplex {
    _write(chunk, _encoding, done) {
        this.emit('data', chunk);
        done();
    }
    _read(_size) { }
}
const workspacePath = process.cwd();
const workspaceUri = vscode_uri_1.default.file(workspacePath).toString();
function prepareClientConnection() {
    return __awaiter(this, void 0, void 0, function* () {
        const up = new TestStream();
        const down = new TestStream();
        const logger = new NullLogger();
        const clientConnection = vscode_languageserver_protocol_1.createProtocolConnection(new vscode_languageserver_protocol_1.StreamMessageReader(down), new vscode_languageserver_protocol_1.StreamMessageWriter(up), logger);
        const serverConnection = vscode_languageserver_1.createConnection(new vscode_languageserver_protocol_1.StreamMessageReader(up), new vscode_languageserver_protocol_1.StreamMessageWriter(down));
        const vls = new vls_1.VLS(serverConnection);
        serverConnection.onInitialize((params) => __awaiter(this, void 0, void 0, function* () {
            yield vls.init(params);
            console.log('Vetur initialized');
            return {
                capabilities: vls.capabilities
            };
        }));
        vls.listen();
        clientConnection.listen();
        const init = Object.assign({ rootPath: workspacePath, rootUri: workspaceUri }, initParams_1.params);
        yield clientConnection.sendRequest(vscode_languageserver_protocol_1.InitializeRequest.type, init);
        return clientConnection;
    });
}
function getDiagnostics() {
    return __awaiter(this, void 0, void 0, function* () {
        const clientConnection = yield prepareClientConnection();
        const files = glob.sync('**/*.vue', { cwd: workspacePath });
        const absFilePaths = files.map(f => path.resolve(workspacePath, f));
        console.log('');
        for (const absFilePath of absFilePaths) {
            console.log('');
            yield clientConnection.sendNotification(vscode_languageserver_protocol_1.DidOpenTextDocumentNotification.type, {
                textDocument: {
                    languageId: 'vue',
                    uri: vscode_uri_1.default.file(absFilePath).toString(),
                    version: 1,
                    text: fs.readFileSync(absFilePath, 'utf-8')
                }
            });
            try {
                const res = (yield clientConnection.sendRequest('$/getDiagnostics', {
                    uri: vscode_uri_1.default.file(absFilePath).toString()
                }));
                console.log(`File: ${absFilePath}`);
                res.forEach(d => {
                    console.log(`Diag: ${d.message}`);
                });
            }
            catch (err) {
                console.log(err);
            }
            console.log('');
        }
    });
}
function getFormattingOutput(print) {
    return __awaiter(this, void 0, void 0, function* () {
        const clientConnection = yield prepareClientConnection();
        const files = glob.sync('**/*.vue', { cwd: workspacePath });
        const absFilePaths = files.map(f => path.resolve(workspacePath, f));
        console.log('');
        for (const absFilePath of absFilePaths) {
            console.log('');
            const vueFileText = fs.readFileSync(absFilePath, 'utf-8');
            const vueFileUri = vscode_uri_1.default.file(absFilePath).toString();
            yield clientConnection.sendNotification(vscode_languageserver_protocol_1.DidOpenTextDocumentNotification.type, {
                textDocument: {
                    languageId: 'vue',
                    uri: vueFileUri,
                    version: 1,
                    text: vueFileText
                }
            });
            const res = yield clientConnection.sendRequest(vscode_languageserver_protocol_1.DocumentFormattingRequest.type, {
                textDocument: {
                    uri: vscode_uri_1.default.file(absFilePath).toString()
                }
            });
            const doc = vscode_languageserver_protocol_1.TextDocument.create(vueFileUri, 'vue', 1, vueFileText);
            const formattedText = res ? vscode_languageserver_protocol_1.TextDocument.applyEdits(doc, res) : vueFileText;
            if (print) {
                fs.writeFileSync(absFilePath, formattedText);
                console.log(`File: ${absFilePath} rewritten`);
            }
            else {
                console.log(`File: ${absFilePath}`);
                console.log(formattedText);
            }
            console.log('');
        }
    });
}
const myArgs = process.argv.slice(2);
// vls diag
if (myArgs[0] === 'diag') {
    console.log('Getting Vetur diagnostics');
    console.log('====================================');
    getDiagnostics().then(() => {
        console.log('====================================');
    });
}
else if (myArgs[0] === 'format') {
    console.log('Getting Vetur diagnostics');
    console.log('====================================');
    const print = myArgs[1] === '-w' || myArgs[1] === '--write';
    getFormattingOutput(print).then(() => {
        console.log('====================================');
    });
}
//# sourceMappingURL=cli.js.map